==> fetch.php?media=programming%3Apython%3Adictdefence-0.1.1.tar.gz: dictdefence-0.1.1/dictdeflib/SQLiteDB.py <==
#!/usr/bin/env python

"""
The class which defines the the methods for reading and updating
and SQLite database
"""

__cvsversion__ = '$Id: SQLiteDB.py,v 1.14 2007/12/06 23:32:20 jay Exp $'
__author__ = 'Jay Deiman; jay@splitstreams.com'
__copyright__ = 'Copyright 2007 Jason Deiman'
__license__ = 'GPLv3'
__all__ = ['SQLiteDB']

from pysqlite2 import dbapi2 as sqlite
import utils , time , DB , os

class SQLiteDB(DB.DB):
    """
    Implements the DB class for sqlite
    """
    def __init__(self , config , actionQ , fwQ , reqQ , respQ , logger):
        DB.DB.__init__(self , config , actionQ , fwQ , reqQ , respQ , logger)
        
    def _cleanup(self):
        """
        Cleanup the database accesses table.  Also unban expired bans.
        """
        # First we remove anything in the accesses table older than
        # sixty days
        sixtyDaysAgo = time.time() - (60 * 60 * 24 * 60)
        delete = 'DELETE FROM accesses WHERE ts < ?'
        self._cursor.execute(delete , (sixtyDaysAgo,))
        self._conn.commit()
        # Now we unban any expired bans
        expireTime = time.time() - self._config.BlockTime
        select = 'SELECT ip_str FROM banned WHERE ts < ?'
        self._cursor.execute(select , (expireTime,))
        for row in self._cursor:
            self.unban(row[0])        
    
    def _initDB(self):
        """
        Initialize the connection to the database and create the
        database if it does not exist
        """
        conn = None
        exists = os.path.exists(self._config.DBFile)
        try:
            conn = sqlite.connect(self._config.DBFile)
        except pysqlite2.dbapi2.OperationalError , e:
            self._logger.log(str(e) , self._logger.LOG_ERR)
            raise pysqlite2.dbapi2.OperationalError , e
        
        if exists:
            return conn
        
        tables = """
            CREATE TABLE accesses (
                acc_id INTEGER PRIMARY KEY ,
                ts DOUBLE ,
                ip_int INTEGER ,
                ip_str VARCHAR(15) ,
                logline VARCHAR(255)
            );
            CREATE INDEX ts_acc_idx ON accesses (ts);
            CREATE INDEX ip_int_acc_idx ON accesses (ip_int);
            CREATE TABLE banned (
                ban_id INTEGER PRIMARY KEY ,
                ip_int INTEGER ,
                ts DOUBLE ,
                ip_str VARCHAR(15) ,
                permanent BOOLEAN
            );
            CREATE INDEX ts_ban_idx ON banned (ts);
            CREATE INDEX ip_int_ban_idx ON banned (ip_int);
            CREATE TABLE prev_banned (
                ban_id INTEGER PRIMARY KEY ,
                ts DOUBLE ,
                ip_int INTEGER ,
                ip_str VARCHAR(15)
            );
            CREATE INDEX ip_int_pban_idx ON prev_banned (ip_int);
        """
        
        curs = conn.cursor()
        # Create the tables
        curs.executescript(tables)
        curs.close()
        return conn
    
    def add(self , ip ,  logLine):
        """
        Adds an item to the database and passes the info to the
        firewall Queue if necessary
        """
        ipInt = utils.ip2int(ip)
        # Insert the access
        ins = (time.time() , ipInt , ip , logLine)
        insert = """
            INSERT INTO accesses (acc_id , ts , ip_int , ip_str , logline)
            VALUES (NULL , ? , ? , ? , ?)
        """
        self._cursor.execute(insert , ins)
        self._conn.commit()
        # Now check if this IP should be banned
        sel = (time.time() - self._config.TimePeriod , ipInt)
        select = 'SELECT COUNT(*) FROM accesses WHERE ts > ? AND ip_int = ?'
        self._cursor.execute(select , sel)
        count = self._cursor.fetchall()[0][0]
        if count >= self._config.NumFails:
            # Ban them
            self.ban(ip)
    
    def isBanned(self , ip):
        """
        Returns a boolean value stating whether the given IP is
        banned
        """
        ipInt = utils.ip2int(ip)
        select = 'SELECT COUNT(*) FROM banned WHERE ip_int = ?'
        self._cursor.execute(select , (ipInt,))
        if int(self._cursor.fetchall()[0][0]) > 0:
            return True
        return False
    
    def ban(self , ip):
        """
        This will ban a given IP by adding it to the database and
        queuing an entry for the firewall thread.  This will also
        check to see if the given IP needs to be permanently
        banned as well.
        """
        ipInt = utils.ip2int(ip)
        # Check to see if this has been banned before
        sel = (ipInt,)
        select = 'SELECT COUNT(*) FROM prev_banned WHERE ip_int = ?'
        self._cursor.execute(select , sel)
        count = int(self._cursor.fetchall()[0][0]) + 1
        if count >= self._config.NumFails:
            # We ban permanently if its over the numfails
            self.banPermanent(ip)
        else:
            # Check to see if the ip is already banned
            select = 'SELECT COUNT(*) FROM banned where ip_int = ?'
            self._cursor.execute(select , sel)
            if int(self._cursor.fetchall()[0][0]) > 0:
                self._logger.log('IP already banned: %s' % ip)
                return
            # Or else we just do a temporary ban
            ins = (ipInt , time.time() , ip)
            insert = """
                INSERT INTO banned (ip_int , ts , ip_str , permanent)
                VALUES (? , ? , ? , 0)
            """
            self._cursor.execute(insert , ins)
            self._conn.commit()
            self._fwQ.put(('ban' , ip))
    
    def unban(self , ip):
        """
        This will unban an IP, moving the entry into the prev_banned
        table and putting an unban request in the firewall queue.
        """
        intIp = utils.ip2int(ip)
        # First delete from the banned table
        delete = 'DELETE FROM banned WHERE ip_int = ?'
        self._cursor.execute(delete , (intIp,))
        # Now insert into the previously banned table
        ins = (time.time() , intIp , ip)
        insert = """
            INSERT INTO prev_banned (ban_id , ts , ip_int , ip_str)
            VALUES (NULL , ? , ? , ?)
        """
        self._cursor.execute(insert , ins)
        self._conn.commit()
        # Add an unban to firewall queue
        self._fwQ.put(('unban' , ip))
    
    def banPermanent(self , ip):
        """
        Bans an IP with the permanent flag set
        """
        ins = (utils.ip2int(ip) , time.time() , ip)
        insert = """
            INSERT INTO banned (ip_int , ts , ip_str , permanent)
            VALUES (? , ? , ? , 1)
        """
        self._cursor.execute(insert , ins)
        self._conn.commit()
        self._fwQ.put(('ban' , 'ip'))
    
    def getBanned(self):
        """
        Returns a list of hashes of data
        """
        data = []
        select = 'SELECT * FROM banned ORDER BY ts DESC'
        self._cursor.execute(select)
        colNames = [desc[0] for desc in self._cursor.description]
        for row in self._cursor:
            data.append(dict(zip(colNames , row)))
        return data
    
    def getAccesses(self):
        """
        Returns a list of hashes of data
        """
        data = []
        select = 'SELECT * FROM accesses ORDER BY ts DESC'
        self._cursor.execute(select)
        colNames = [desc[0] for desc in self._cursor.description]
        for row in self._cursor:
            data.append(dict(zip(colNames , row)))
        return data
    
    def getPrevBanned(self):
        """
        Returns a list of hashes of data
        """
        data = []
        select = 'SELECT * FROM prev_banned ORDER BY ts DESC'
        self._cursor.execute(select)
        colNames = [desc[0] for desc in self._cursor.description]
        for row in self._cursor:
            data.append(dict(zip(colNames , row)))
        return data
    
    def __del__(self):
        """
        Destructor: cleans up the database connections.
        """
        self._cursor.close()
        self._conn.close()

==> fetch.php?media=programming%3Apython%3Adictdefence-0.2.tar.gz: dictdefence-0.2/dictdeflib/SQLiteDB.py <==
#!/usr/bin/env python

"""
The class which defines the the methods for reading and updating
and SQLite database
"""

__cvsversion__ = '$Id: SQLiteDB.py,v 1.15 2007/12/14 17:33:00 jay Exp $'
__author__ = 'Jay Deiman; jay@splitstreams.com'
__copyright__ = 'Copyright 2007 Jason Deiman'
__license__ = 'GPLv3'
__all__ = ['SQLiteDB']

from pysqlite2 import dbapi2 as sqlite
import utils , time , DB , os

class SQLiteDB(DB.DB):
    """
    Implements the DB class for sqlite
    """
    def __init__(self , config , actionQ , fwQ , reqQ , respQ , logger):
        DB.DB.__init__(self , config , actionQ , fwQ , reqQ , respQ , logger)
        
    def _cleanup(self):
        """
        Cleanup the database accesses table.  Also unban expired bans.
        """
        # First we remove anything in the accesses table older than
        # sixty days
        sixtyDaysAgo = time.time() - (60 * 60 * 24 * 60)
        delete = 'DELETE FROM accesses WHERE ts < ?'
        self._cursor.execute(delete , (sixtyDaysAgo,))
        self._conn.commit()
        # Now we unban any expired bans
        expireTime = time.time() - self._config.BlockTime
        select = 'SELECT ip_str FROM banned WHERE ts < ?'
        self._cursor.execute(select , (expireTime,))
        for row in self._cursor:
            self.unban(row[0])        
    
    def _initDB(self):
        """
        Initialize the connection to the database and create the
        database if it does not exist
        """
        conn = None
        exists = os.path.exists(self._config.DBFile)
        try:
            conn = sqlite.connect(self._config.DBFile)
        except pysqlite2.dbapi2.OperationalError , e:
            self._logger.log(str(e) , self._logger.LOG_ERR)
            raise pysqlite2.dbapi2.OperationalError , e
        
        if exists:
            return conn
        
        tables = """
            CREATE TABLE accesses (
                acc_id INTEGER PRIMARY KEY ,
                ts DOUBLE ,
                ip_int INTEGER ,
                ip_str VARCHAR(15) ,
                logline VARCHAR(255)
            );
            CREATE INDEX ts_acc_idx ON accesses (ts);
            CREATE INDEX ip_int_acc_idx ON accesses (ip_int);
            CREATE TABLE banned (
                ban_id INTEGER PRIMARY KEY ,
                ip_int INTEGER ,
                ts DOUBLE ,
                ip_str VARCHAR(15) ,
                permanent BOOLEAN
            );
            CREATE INDEX ts_ban_idx ON banned (ts);
            CREATE INDEX ip_int_ban_idx ON banned (ip_int);
            CREATE TABLE prev_banned (
                ban_id INTEGER PRIMARY KEY ,
                ts DOUBLE ,
                ip_int INTEGER ,
                ip_str VARCHAR(15)
            );
            CREATE INDEX ip_int_pban_idx ON prev_banned (ip_int);
        """
        
        curs = conn.cursor()
        # Create the tables
        curs.executescript(tables)
        curs.close()
        return conn
    
    def add(self , ip ,  logLine):
        """
        Adds an item to the database and passes the info to the
        firewall Queue if necessary
        """
        ipInt = utils.ip2int(ip)
        # Insert the access
        ins = (time.time() , ipInt , ip , logLine)
        insert = """
            INSERT INTO accesses (acc_id , ts , ip_int , ip_str , logline)
            VALUES (NULL , ? , ? , ? , ?)
        """
        self._cursor.execute(insert , ins)
        self._conn.commit()
        # Now check if this IP should be banned
        sel = (time.time() - self._config.TimePeriod , ipInt)
        select = 'SELECT COUNT(*) FROM accesses WHERE ts > ? AND ip_int = ?'
        self._cursor.execute(select , sel)
        count = self._cursor.fetchall()[0][0]
        if count >= self._config.NumFails:
            # Ban them
            self.ban(ip)
    
    def isBanned(self , ip):
        """
        Returns a boolean value stating whether the given IP is
        banned
        """
        ipInt = utils.ip2int(ip)
        select = 'SELECT COUNT(*) FROM banned WHERE ip_int = ?'
        self._cursor.execute(select , (ipInt,))
        if int(self._cursor.fetchall()[0][0]) > 0:
            return True
        return False
    
    def ban(self , ip):
        """
        This will ban a given IP by adding it to the database and
        queuing an entry for the firewall thread.  This will also
        check to see if the given IP needs to be permanently
        banned as well.
        """
        ipInt = utils.ip2int(ip)
        # Check to see if this has been banned before
        sel = (ipInt,)
        select = 'SELECT COUNT(*) FROM prev_banned WHERE ip_int = ?'
        self._cursor.execute(select , sel)
        prevBCount = int(self._cursor.fetchall()[0][0]) + 1
        perm = prevBCount >= self._config.NumFails
        if perm:
            # We ban permanently if its over the numfails
            self.banPermanent(ip)
        else:
            # Check to see if the ip is already banned
            select = 'SELECT COUNT(*) FROM banned where ip_int = ?'
            self._cursor.execute(select , sel)
            if int(self._cursor.fetchall()[0][0]) > 0:
                self._logger.log('IP already banned: %s' % ip)
                return
            # Or else we just do a temporary ban
            ins = (ipInt , time.time() , ip)
            insert = """
                INSERT INTO banned (ip_int , ts , ip_str , permanent)
                VALUES (? , ? , ? , 0)
            """
            self._cursor.execute(insert , ins)
            self._conn.commit()
            self._fwQ.put(('ban' , ip))
        if self._notifier:
            accesses = self.getAccesses(ip)
            try:
                self._notifier.notifyBan(ip , accesses , prevBCount , perm)
            except Exception , e:
                self._logger.log('Error during email notification: %s' % e)
    
    def unban(self , ip):
        """
        This will unban an IP, moving the entry into the prev_banned
        table and putting an unban request in the firewall queue.
        """
        intIp = utils.ip2int(ip)
        # First delete from the banned table
        delete = 'DELETE FROM banned WHERE ip_int = ?'
        self._cursor.execute(delete , (intIp,))
        # Now insert into the previously banned table
        ins = (time.time() , intIp , ip)
        insert = """
            INSERT INTO prev_banned (ban_id , ts , ip_int , ip_str)
            VALUES (NULL , ? , ? , ?)
        """
        self._cursor.execute(insert , ins)
        self._conn.commit()
        # Add an unban to firewall queue
        self._fwQ.put(('unban' , ip))
    
    def banPermanent(self , ip):
        """
        Bans an IP with the permanent flag set
        """
        ins = (utils.ip2int(ip) , time.time() , ip)
        insert = """
            INSERT INTO banned (ip_int , ts , ip_str , permanent)
            VALUES (? , ? , ? , 1)
        """
        self._cursor.execute(insert , ins)
        self._conn.commit()
        self._fwQ.put(('ban' , 'ip'))
    
    def getBanned(self):
        """
        Returns a list of hashes of data
        """
        data = []
        select = 'SELECT * FROM banned ORDER BY ts DESC'
        self._cursor.execute(select)
        colNames = [desc[0] for desc in self._cursor.description]
        for row in self._cursor:
            data.append(dict(zip(colNames , row)))
        return data
    
    def getAccesses(self , ip=None):
        """
        Returns a list of hashes of data
        """
        data = []
        err = False
        if ip:
            try:
                ipInt = utils.ip2int(ip)
            except:
                err = True
            else:
                select = 'SELECT * FROM accesses WHERE ip_int = ? ' + \
                         'ORDER BY ts DESC'
                self._cursor.execute(select , (ipInt,))
        if not ip or err:
            select = 'SELECT * FROM accesses ORDER BY ts DESC'
            self._cursor.execute(select)
        colNames = [desc[0] for desc in self._cursor.description]
        for row in self._cursor:
            data.append(dict(zip(colNames , row)))
        return data
    
    def getPrevBanned(self):
        """
        Returns a list of hashes of data
        """
        data = []
        select = 'SELECT * FROM prev_banned ORDER BY ts DESC'
        self._cursor.execute(select)
        colNames = [desc[0] for desc in self._cursor.description]
        for row in self._cursor:
            data.append(dict(zip(colNames , row)))
        return data
    
    def __del__(self):
        """
        Destructor: cleans up the database connections.
        """
        self._cursor.close()
        self._conn.close()
